#!/usr/bin/perl
###########################
##  AutoRank Pro v3.0.x  ##
##################################################
##  setup.cgi - setup variables/options         ##
##################################################

package setup;

use lib '..';                                      ## You may need to set this to the full path to cgiworks.pm
use strict;
use cgiworks;

print "Content-type: text/html\n\n";
$HEADER = 1;

eval {
  require "$DDIR/errors.dat";
  require "$DDIR/help.dat";
  require "$DDIR/def.html";
  require 'arp.pl';
  main();
};

err("$@", 'setup.cgi') if( $@ );
exit;

#####################################################################
##  Removing the link back to CGI Works is a copyright violation.  ##
##  Altering or removing any of the code that is responsible, in   ##
##  any way, for generating that link is strictly forbidden.       ##
##  Anyone violating the above policy will have their license      ##
##  terminated on the spot.  Do not remove that link - ever.       ##
#####################################################################

sub main {
  derr(1000, '.htaccess Validation Required') if( !$RMTUSR );
  if( $REQMTH eq 'GET' ) {
    fwritenew("$DDIR/admin.epf", crypt('admin', getsalt()));
    display_main();
  }
  else {
    parsepost();
    save_data();
  }
}

sub display_main {
  my @opt = qw( OPT_GSP OPT_BCR OPT_OUT OPT_REV OPT_EML OPT_ADM OPT_EDT OPT_CLG OPT_ILG OPT_GAT OPT_COK OPT_RCP OPT_LWP OPT_CHU OPT_NEW OPT_CRN OPT_FD1 OPT_FD2 OPT_FD3 OPT_FCP );
  my $vars = freadall("$DDIR/vars.dat") if( -e "$DDIR/vars.dat" );

  for( @{ $vars } ) {
    $_ =~ m/\$([^\s]+) = '([^']*)'/;
    my($vrbl, $val) = ($1, $2);
    if( $vrbl ne "" ) {
      $vrbl =~ s/::/_/;
      $TPL{$vrbl} = $val;
    }
  }

  for( @opt ) {
    $TPL{$_} = getcb( $_, $TPL{$_} );
  }

  $TPL{'VAR_F1'} = 'Field One' if( $TPL{'VAR_F1'} eq '' );
  $TPL{'VAR_F2'} = 'Field Two' if( $TPL{'VAR_F2'} eq '' );
  $TPL{'VAR_F3'} = 'Field Three' if( $TPL{'VAR_F3'} eq '' );
  $TPL{'VAR_RB'} = getropt();
  fparse('_admin_setup.htmlt');
}

sub check_input {
  my @req = qw( VAR_HD VAR_CU VAR_FU VAR_IU VAR_OU VAR_ES VAR_EM VAR_RP VAR_OL VAR_MH VAR_MT VAR_MD VAR_RR VAR_RS VAR_HU VAR_BH VAR_BW );
  push( @req, qw( VAR_CT VAR_FE VAR_CL VAR_CB )) if( $FRM{'OPT_GSP'} );
  for( @req ) {
    derr(1001, "Required Field Left Blank $_") if( $FRM{$_} eq "" );
  }

  derr(1024, "Invlid Space Found") if( $FRM{'VAR_CT'} =~ /,\s|\s,|\s,\s/ );
}

sub save_data {
  check_input();

  dcreate("$DDIR/members", 0777);
  dcreate("$DDIR/breaks", 0777);
  dcreate("$DDIR/dbs", 0777);
  dcreate("$DDIR/icons", 0777);
  dcreate("$DDIR/temp", 0777);
  dcreate("$DDIR/ips", 0777);
  dcreate("$DDIR/mails", 0777);
  dcreate("$DDIR/html", 0777);
  dcreate("$DDIR/times", 0777);
  dcreate("$DDIR/ses", 0777);

  fcreate("$DDIR/dbs/email.ban");
  fcreate("$DDIR/dbs/url.ban");
  fcreate("$DDIR/dbs/word.ban");
  fcreate("$DDIR/dbs/review.db");
  fcreate("$DDIR/dbs/sites");
  fcreate("$DDIR/dbs/search");
  
  fwritenew("$DDIR/dbs/bdata", pack('u', $err::code{'1077'}));
  fwritenew("$DDIR/dbs/start", time + ($FRM{'VAR_TZ'} * 3600) . "\n");

  fcreate("$DDIR/times/rerank");
  fcreate("$DDIR/times/reset");
  fcreate("$DDIR/times/backup");
  fcreate("$DDIR/times/clean");

  fcreate("$DDIR/error.log");
  fcreate("$DDIR/cheat.log");
  fcreate("$DDIR/stats");

  $FRM{'VAR_F1'} = 'Field One' if( $FRM{'VAR_F1'} eq '' );
  $FRM{'VAR_F2'} = 'Field Two' if( $FRM{'VAR_F2'} eq '' );
  $FRM{'VAR_F3'} = 'Field Three' if( $FRM{'VAR_F3'} eq '' );

  for( 1..255 ) {
    fcreate("$DDIR/ips/$_");
  }

  $FRM{'VAR_PO'} = $FRM{'VAR_RP'};
  my $def = freadalls("$DDIR/def.html");
  fwritenew("$DDIR/html/$FRM{'VAR_RP'}",  $$def);
  my($nm, @fe) = split(/\./, $FRM{'VAR_RP'});

  for( 2..ssize(\$FRM{'VAR_SO'}, ',') ) {
    fwritenew("$DDIR/html/$nm$_." . join('.', @fe), $$def );
    $FRM{'VAR_PO'} .= ",$nm$_." . join('.', @fe);
  }

  for( split(/,/, $FRM{'VAR_CT'}) ) {
    my $fn = arp::htmlname($_);
    fwritenew("$DDIR/html/$fn.$FRM{'VAR_FE'}", $$def ) if( $FRM{'OPT_GSP'} );
    $FRM{'VAR_PC'} .= ",$fn.$FRM{'VAR_FE'}";
  }

  my $start = 1;
  my @split = split(/,/, $FRM{'VAR_SO'});
  push(@split, $FRM{'VAR_OL'});
  my $hash;

  for( split(/,/, $FRM{'VAR_PO'}) ) {
    my $end = shift(@split);
    $end = $start + 1 if( $end <= $start );
    $hash .= "'$_' => '$start-$end', ";
    $start = $end + 1;
  }
  
  my $font;
  for( split(/,/, $FRM{'VAR_FO'}) ) {
    my $text = $_;
    $text =~ s/\-\d+//;
    $font .= "$text, "; 
  }

  my $cfont;
  for( split(/,/, $FRM{'VAR_FC'}) ) {
    my $text = $_;
    $text =~ s/\-\d+//;
    $cfont .= "$text, "; 
  }

  $hash =~ s/, $//;
  $font =~ s/, $//;
  $cfont =~ s/, $//;
  $FRM{'VAR_PC'} =~ s/^,//;
  open(VARS, ">$DDIR/vars.dat") || err($!, "vars.dat");
  for( sort keys %FRM ) {
    my $key = $_;
    $key =~ s/_/::/;
    $FRM{$_} =~ s/'//g;
    $FRM{$_} =~ s/\r//g;
    $FRM{$_} =~ s/\n//g;
    $FRM{$_} =~ s/\/$//g;
    print VARS "\$$key = '$FRM{$_}';\n" if( $_ =~ /_/ );
  }
  print VARS "%VAR::SP = ($hash);\n";
  print VARS "%VAR::FT = ($font);\n";
  print VARS "%VAR::TC = ($cfont);\n";
  print VARS "1;\n";

  mode(0666, "$DDIR/vars.dat");

  $TPL{'SDDIR'} = dirtest($DDIR);
  $TPL{'MHDIR'} = dirtest($FRM{'VAR_HD'});
  $TPL{'DATE'}  = fdate($FRM{'VAR_DF'});
  $TPL{'TIME'}  = ftime($FRM{'VAR_TF'});

  fparse('_admin_verify.htmlt');
}

sub getcb {
  my($item, $value) = @_;
  $item =~ s/::/_/;
  return qq|<input type="checkbox" name="$item" value="1" checked>| if( $value eq '1');
  return qq|<input type="checkbox" name="$item" value="1">|;
}

sub dirtest {
  my $dir = shift;  
  open(FILE, ">$dir/test.file") || return "<font color='red'>Failed!</font><br>Could Not Create File<br>$!";
  print FILE "TEST PASSED!";
  close(FILE);
  
  unlink("$dir/test.file") || return "<font color='red'>Failed!</font><br>Could Not Remove File<br>$!";
  
  return "<font color='blue'>Passed</font>";
}

sub getropt {
  my %rank = ('srt::byin', 'Hits In', 'srt::byout"', 'Hits Out', 'srt::bytin', 'Total Hits In', 'srt::bytout', 'Total Hits Out', 'srt::byidav', 'Hits In/Day', 'srt::byiwav', 
              'Hits In/Week', 'srt::byimav', 'Hits In/Month', 'srt::byodav', 'Hits Out/Day', 'srt::byowav', 'Hits Out/Week', 'srt::byomav', 'Hits Out/Month');

  my $html = '';
  for( sort keys %rank ) {
    $html .= qq|<option value="$_"|;
    $html .= qq| selected| if( $VAR::RB eq $_ );
    $html .= qq|>$rank{$_}</option>\n|;
  }
  return $html;
}